<?php

namespace Drupal\api_tokens_example\Plugin\ApiToken;

use Drupal\api_tokens\Plugin\ApiTokenBase;
use Drupal\block\Entity\Block as EntityBlock;

/**
 * Provides Block API token.
 *
 * Token examples:
 * - [api:block["bartik_breadcrumbs"]/].
 *
 * @ApiToken(
 *   id = "block",
 *   title = @Translation("Block"),
 *   description = @Translation("Renders a block.")
 * )
 */
class Block extends ApiTokenBase {

  /**
   * {@inheritdoc}
   */
  public function validate(array $params, array $args) {
    // For [api:block["bartik_breadcrumbs"]/] token:
    //$params = [
    //  0 => 'bartik_breadcrumbs',
    //];
    //$args = [
    //  'block_id' => 'bartik_breadcrumbs',
    //];

    // Check that "block_id" is a string.
    if (!is_string($args['block_id'])) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function fallback() {
    // Cut out the API token on fallback.
    return '';
  }

  /**
   * Build callback.
   *
   * @param string $block_id
   *   The block ID.
   *
   * return array
   *   A renderable array.
   *
   * @see \Drupal\api_tokens\Plugin\ApiTokenInterface::_build();
   */
  public function build($block_id) {
    $block = EntityBlock::load($block_id);
    if ($block) {
      $build = \Drupal::entityManager()->getViewBuilder('block')->view($block);
    }
    else {
      $build = $this->getFallback();
    }

    return $build;
  }

}
