<?php

namespace Drupal\api_tokens_example\Plugin\ApiToken;

use Drupal\api_tokens\Plugin\ApiTokenBase;

/**
 * Provides a Date API token.
 *
 * Token examples:
 * - [api:date["D, d M y H:i:s"]/].
 *
 * @ApiToken(
 *   id = "date",
 *   title = @Translation("Date"),
 *   description = @Translation("Renders the current date.")
 * )
 */
class Date extends ApiTokenBase {

  /**
   * {@inheritdoc}
   */
  //protected $cacheMaxAge = 0;

  /**
   * {@inheritdoc}
   */
  public function validate(array $params, array $args) {
    // For [api:date/] token:
    //$params = [];
    //$args = [
    //  'format' => 'U',
    //];

    // For [api:date["D, d M y H:i:s"]/] token:
    //$params = [
    //  0 => 'D, d M y H:i:s',
    //];
    //$args = [
    //  'format' => 'D, d M y H:i:s',
    //];

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function fallback() {
    // Cut out the API token on fallback.
    return '';
  }

  /**
   * Build callback.
   *
   * There are several ways to make the token output uncacheable:
   * - call \Drupal\api_tokens\Plugin\ApiTokenInterface::mergeCacheMaxAge(0);
   * - add ['#cache']['max-age'] = 0; cacheability metadata to the build array;
   * - override plugin's $cacheMaxAge property to 0.
   *
   * @param string $format
   *   (optional) The date format.
   *
   * return array
   *   A renderable array.
   *
   * @see \Drupal\api_tokens\Plugin\ApiTokenInterface::_build();
   */
  public function build($format = 'U') {
    $build = [
      '#markup' => date($format),
      '#prefix' => '<span class="api-token-date">',
      '#suffix' => '</span>',
      '#attached' => [
        'library' => [
          'api_tokens_example/date',
        ],
      ],
      //'#cache' => [
      //  '#max-age' => 0,
      //],
    ];
    $this->mergeCacheMaxAge(0);

    return $build;
  }

}
