<?php

namespace Drupal\api_tokens_example\Plugin\ApiToken;

use Drupal\api_tokens\Plugin\ApiTokenBase;
use Drupal\Core\Cache\CacheableMetadata;

/**
 * Provides Node API token.
 *
 * Token examples:
 * - [api:node[123]/];
 * - [api:node[123, "teaser"]/].
 *
 * @ApiToken(
 *   id = "node",
 *   title = @Translation("Node"),
 *   description = @Translation("Renders a node.")
 * )
 */
class Node extends ApiTokenBase {

  /**
   * {@inheritdoc}
   */
  public function validate(array $params, array $args) {
    // For [api:node[123]/] token:
    //$params = [
    //  0 => 123,
    //];
    //$args = [
    //  'nid' => 123,
    //  'view_mode' => 'full',
    //];

    // For [api:node[123, "teaser"]/] token:
    //$params = [
    //  0 => 123,
    //  1 => 'teaser',
    //];
    //$args = [
    //  'nid' => 123,
    //  'view_mode' => 'teaser',
    //];

    // Check that "nid" is a valid node ID.
    if (!ctype_digit((string) $args['nid'])) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function fallback() {
    $message = $this->t($this->recursion ? 'RECURSION' : 'Node can not be rendered');
    return "****$message****";
  }

  /**
   * Build callback.
   *
   * @param int $nid
   *   The node ID.
   * @param string $view_mode
   *   (optional) The view mode to render a node in. Defaults to "full".
   *
   * return array
   *   A renderable array.
   *
   * @see \Drupal\api_tokens\Plugin\ApiTokenInterface::_build();
   */
  public function build($nid, $view_mode = 'full') {
    $node = \Drupal::entityTypeManager()->getStorage('node')->load($nid);
    if ($node && $node->access('view')) {
      $build = \Drupal::entityTypeManager()->getViewBuilder('node')->view($node, $view_mode);
      CacheableMetadata::createFromRenderArray($build)->addCacheContexts(['url.path'])->applyTo($build);
    }
    else {
      $build = $this->getFallback();
    }
    $this->addCacheTags(['some_cache_tag']);

    return $build;
  }

}
