<?php

namespace Drupal\api_tokens_example\Plugin\ApiToken;

use Drupal\api_tokens\Plugin\ApiTokenBase;
use Drupal\Core\Link;

/**
 * Provides User Link API token.
 *
 * Token examples:
 * - [api:user-link/];
 * - [api:user-link[123]/].
 *
 * @ApiToken(
 *   id = "user-link",
 *   title = @Translation("User link"),
 *   description = @Translation("Renders a user link.")
 * )
 */
class UserLink extends ApiTokenBase {

  /**
   * {@inheritdoc}
   */
  public function validate(array $params, array $args) {
    // For [api:user-link/] token:
    //$params = [];
    //$args = [
    //  'uid' => NULL,
    //];

    // For [api:user-link[123]/] token:
    //$params = [
    //  0 => 123,
    //];
    //$args = [
    //  'uid' => 123,
    //];

    // If "uid" is provided, check it is a valid user ID.
    if ($params && !ctype_digit((string) $args['uid'])) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function fallback() {
    // Leave the API token unchanged on fallback.
    return $this->getToken();
  }

  /**
   * Build callback.
   *
   * @param int $uid
   *   (optional) The user ID.
   *
   * return array
   *   A renderable array.
   *
   * @see \Drupal\api_tokens\Plugin\ApiTokenInterface::_build();
   */
  public function build($uid = NULL) {
    $link = NULL;
    if ($uid) {
      $user = \Drupal::entityTypeManager()->getStorage('user')->load($uid);
      if ($user && $user->access('view')) {
        $link = Link::createFromRoute($user->getDisplayName(), 'entity.user.canonical', [
          'user' => $uid,
        ]);
        $this->addCacheContexts(['user']);
        $this->addCacheableDependency($user);
      }
    }
    else {
      $user = \Drupal::currentUser();
      $link = Link::createFromRoute($user->getDisplayName(), 'user.page');
      $this->addCacheContexts(['user']);
    }
    $build = $link ? $link->toRenderable() : $this->getFallback();

    return $build;
  }

}
